; $LastChangedDate: 2008-08-17 11:08:59 -0500 (Sun, 17 Aug 2008) $
; $LastChangedRevision: 2727 $
; $HeadURL: svn://osiris/scheme-bos-clug/tags/2.01/bos-pres.scm $
;
; I prepared this presentation for the 2nd Chicago Lisp User Group meeting.
; It started out as a homework exercise about objects; and quickly became
; an opportunity for me to dig in to hygienic macros.
#lang slideshow

(require (lib "code.ss" "slideshow")
         (lib "run.ss" "slideshow")
         "../grant/slideshow-helper.scm")

(add-slide 
 1
 (define (slide-1)
   (slide/title/center
    "A Basic Object System Using Macros"
    (page-para* "A Talk With Grant Rettke")
    (page-para* "")
    (page-para* "grettke@acm.org")
    (page-para* "")
    (page-para* "http://www.wisdomandwonder.com/"))))

(add-slide 
 2
 (define (slide-2)
   (slide/title
    "How To Roll A Basic Object System"
    'next 
    (page-item "Features")
    'next      
    (page-subitem "Public Methods, Private Variables, No Inheritance (Simple huh?!)")
    'next
    (page-item "Goals")
    'next
    (page-subitem "Study Scheme, Macros, Language Constructs")
    'next
    (page-subitem "Chat About It With My Friends")
    'next
    (page-item "Approach")
    'next
    (page-subitem "Code First, Then Generation")   
    'next
    (page-subitem "Simplicity Trumps Efficiency")
    'next
    (page-subitem "No Mystery Code!"))))

(add-slide 
 3
 (define (slide-3)
   (slide/title
    "Destination Code Features"
    'next
    (page-item "Primitive Object Implementation")
    'next
    (page-item "Methods and Variables")
    'next
    (page-item "Encapsulation")
    'next
    (page-item "Message Passing")
    'next
    (page-item "Duplicate variable/method name warning")
    'next
    (page-item "Built on \"Stock\" Language Features")
    'next
    (page-item "Reference: 5-prim-obj-stx-smpl.scm, 6-prim-obj-stx-smpl-tsts.scm"))))

(add-slide 
 4
 (define (slide-4)
   (slide/title
    "Destination Code Sample"
    'next
    (code
     (define-object person
       (variables
        ([name #f]
         [age-years #f]
         [method-names-ls2 10]))
       (methods
        ([set-name [arg] (set! name arg)]
         [get-name [] name]
         [set-age-years [arg] (set! age-years arg)]
         [get-age-years [] age-years]      
         [age-in-days [] (* age-years 365)]
         [typed-name [] (cons (get-name) (class-name))]
         [method-names2 [] 'OVERWRITTEN]
         [typed-name2 [] 'OVERWRITTEN]))))
    'next
    (page-item "Brackets may be used anywhere parentheses are used")
    'next
    (page-subitem "Primarily to enhance readability"))))

(add-slide 
 5
 (define (slide-5)
   (slide/title
    "Step 1"
    'next
    (page-item "Exploring Primitive Language Features")
    'next
    (page-subitem "Object Creation")
    'next
    (page-subitem "Message Passing")
    'next
    (page-subitem "Lexical Scope")
    'next
    (page-item "Reference: 1-prm-feat.scm"))))

(add-slide 
 6
 (define (slide-6)
   (slide/title
    "Object Creation"
    'next
    (code
     (define prim-obj-creation
       (λ () 
         (λ () 
           #t))))
    'next
    (page-item "Goal: An object is a thing that can be instantiated")
    'next
    (page-item "This code is a function that returns a 1st class function")
    'next
    (page-item "A 1st class function is a thing, virtually an \"object\"")
    'next
    (page-item "This is how objects will be instantiated in this system"))))

(add-slide 
 7
 (define (slide-7)
   (slide/title
    "Message Passing"
    'next
    (code
     (define prim-obj-msg-passing
       (λ () 
         (λ (msg) 
           msg))))
    'next
    (page-item "Goal: An object is a thing that can receive a message")
    'next
    (page-item "The first class function above can receive messages")
    'next
    (page-item "This is how \"message-passing\" will occur in this object system"))))

(add-slide 
 8
 (define (slide-8)
   (slide/title
    "Lexical Scope"
    'next
    (code
     (define prim-obj-lex-scope
       (λ () 
         (define x 11)
         (define y 12)
         (define frobnicate
           (λ (a b) 
             (+ a b)))
         (λ (msg) 
           (cons msg (frobnicate x y))))))
    'next 
    (page-item "Goal: Encapsulation, introduction of scope")
    'next 
    (page-item "Lambda introduces lexical scope for internal" (code define) "(aka" (code letrec) ") appearing immediately after it")
    'next 
    (page-item "The 1st class function returned by this function inherits the lexical scope in which it was created (" (code x) "," (code y) ", and" (code frobnicate) ")")
    'next
    (page-item "This is how objects with lexical scope and encapsulation are created"))))

(add-slide 
 9
 (define (slide-9)
   (slide/title
    "prim-obj-lex-scope usage"
    'next
    (code
     (define obj (prim-obj-lex-scope))
     (obj 'my-message))
    'next
    (page-item "-> " (code (my-message . 23)))
    'next
    (page-item "This is how objects may be instantiated and sent messages"))))

(add-slide 
 10
 (define (slide-10)
   (slide/title
    "The Non-Macro Primitive Object"
    'next
    (page-item "Combine those three primitive features to hand-code a primitive object")
    'next
    (page-item "Reference: 3-A-prim-obj.scm"))))

(add-slide 
 11
 (define (slide-11)
   (slide/title
    "The Non-Macro Primitive Object Code Sample"
    'next
    (code
     (define person
       (λ () 
         (define name #f)
         (define age-years #f)        
         (define set-name (λ (arg) (set! name arg)))
         (define get-name (λ () name))
         <methods go here>
         (λ (msg . args) 
           (case msg
             [(set-name) (apply set-name args)]
             [(get-name) (get-name)]
             [(set-age-years) (apply set-age-years args)]
             [(get-age-years) (get-age-years)]
             [(age-in-days) (age-in-days)]
             [else (error "message not understood" msg)])))))
    'next
    (page-item "The last λ expression is the \"object\""))))

(add-slide 
 12
 (define (slide-12)
   (slide/title
    "Next Step: On To Generation"
    'next
    (page-item "High Level Macro Review"))))

(add-slide 
 13
 (define (slide-13)
   (slide/title
    "Macros 1 - Why"
    'next
    (bitmap "SE1.gif")
    'next
    (page-item "Modify input code to produce new output code")
    'next
    (page-item "Seemingly superior to C style pre-processor macros")
    'next
    (page-item "Change shape, and even order of evaluation of, the code"))))

(add-slide 
 14
 (define (slide-14)
   (slide/title
    "Macros 1 - Why - Example"
    'next
    (page-item "In a conditional expression, every clause may not be evaluated")
    'next
    (page-item "Consider a typical if-null-then check (illustrated by the macro" (code my-if) " moving forward)")
    'next
    (code
     (let ([fun null])
       (my-if (null? fun)
              (printf "Can't call fun, it is null~n")
              (printf "x is ~a~n" (fun)))))
    'next
    (page-item (code my-if) "could never be a function because it would evaluate its arguments, resulting in a null pointer")
    'next
    (page-item "Reference: macros.scm"))))

(add-slide 
 15
 (define (slide-15)
   (slide/title
    "Macros 2 - What"
    (bitmap "SE2.gif")
    'next
    (page-item "The object sent to the macro is called a syntax object")
    'next
    (page-item "The macro itself is implemented by an object called a transformer"))))

(add-slide 
 16
 (define (slide-16)
   (slide/title
    "Macros 2 - What - Example"
    'next
    (page-item "Syntax Object (everything enclosed by" (code my-if) ")")
    'next
    (code
     (let ([fun null])
       (my-if (null? fun)
              (printf "Can't call fun, it is null~n")
              (printf "x is ~a~n" (fun)))))
    'next
    (page-item "Transformer (this is the" (code my-if) "macro)")
    'next
    (code
     [(_ clause true-body false-body)
      #'(let ([c clause])
          (if c 
              true-body
              false-body))])
    'next
    (page-item "#' is shorthand for surrounding the following shape inside a call to " (code syntax)))))

(add-slide 
 17
 (define (slide-17)
   (slide/title
    "Macros 3 - How"
    'next
    (page-item "The macro" (code my-if) "takes an input form as its argument")
    'next
    (code
     (my-if (null? fun)
            (printf "Can't call fun, it is null~n")
            (printf "x is ~a~n" (fun))))
    'next
    (page-item "De-structures it using pattern matching into 3 different parts:" (code clause) ", " (code true-body) ", and" (code false-body))
    'next
    (code (my-if clause true-body false-body))
    'next
    (page-item "Defines a template for the new form (the resulting syntax object)")
    'next
    (code
     #'(let ([c clause]) (if c true-body false-body))))))

(add-slide 
 18
 (define (slide-18)
   (slide/title
    "Macros 3 - How"
    'next
    (page-item "Expands the template by replacing the pattern variables with their actual values and environment, and returns the resulting syntax-object")
    'next
    (code
     (my-if (null? fun)
            (printf "Can't call fun, it is null~n")
            (printf "x is ~a~n" (fun))))
    'next
    (page-item "Expanding into")
    'next
    (code
     (let ((c (null? fun))) 
       (if c 
           (printf "Can't call fun, it is null~n") 
           (printf "x is ~a~n" (fun))))))))

(add-slide 
 19
 (define (slide-19)
   (slide/title
    "Macros 3 - How - Visual"
    (bitmap "SE3.gif"))))

(add-slide 
 20
 (define (slide-20)
   (slide/title
    "Two Kinds of Macros"
    'next
    (page-item "Hygienic")
    'next
    (page-subitem "Guarantee that expansion will not redefine existing name bindings")
    'next
    (page-item "Lexical Scope Twisting (Un-Hygienic)")
    'next
    (page-subitem "By design allows you to modify existing bindings")
    'next
    (page-subitem "Why? To introduce a" (code return) " statement, or see \"On Lisp\" Anaphoric Macros")
    'next
    (page-subitem "Anaphora: use of a grammatical substitute (as a pronoun or a pro-verb) to refer to the denotation of a preceding word or group of words."))))

(add-slide 
 21
 (define (slide-21)
   (slide/title
    "Hygienic Macro 'hm' Template Source"
    (code
     [(_ body)
      #'(begin
          (define food 'perch)
          (define utensil 'fork)          
          (printf "~a, ~a~n" food utensil)
          body)])
    'next
    (page-item (code _) "is the first argument of the pattern, and is always ignored." "Using" (code _) "is both loved and hated by Schemers")
    'next
    (page-item (code body) "matches the entire form appearing as the argument to" (code hm))
    'next
    (page-item "Everything following #' is the template"))))

(add-slide 
 22
 (define (slide-22)
   (slide/title
    "Hygienic Macro 'hm' Template Body"
    'next
    (bitmap "SE4.gif")
    'next
    (code
     (let ([food 'salmon]
           [utensil 'spoon])
       (hm 
        (printf 
         "~a, ~a~n" 
         food utensil))))
    'next
    (page-item "Everything following" (code hm) ", along with its environment, is the argument for" (code hm)))))

(add-slide 
 23
 (define (slide-23)
   (slide/title 
    "Hygienic Macro 'hm' Template Expansion" 
    'next
    (code
     (let ([food 'salmon]
           [utensil 'spoon])
       (hm (printf "~a, ~a~n" food utensil))))
    'next
    (page-item "Expands into")
    'next
    (code
     (let([food 'salmon] 
          [utensil 'spoon])
       (begin 
         [define food 'perch]
         [define utensil 'fork] 
         (printf "~a, ~a~n" food utensil) 
         (printf "~a, ~a~n" food utensil))))
    'next
    (page-item "On the next page is the interesting part; the" (code printfs) "still use the correctly bound values"))))

(add-slide 
 24
 (define (slide-24)
   (slide/title
    "Hygienic Macro 'hm' Template Expansion"
    'next
    (page-item "[Review code in macro stepper and tracing arrows]")
    'next
    (page-item "Prints \"perch, fork\", then \"salmon, spoon\""))))

(add-slide 
 25
 (define (slide-25)
   (slide
    ; Commented out to conserve space
    ; "Syntactic Extension 5 - Hygienic Macro Template"
    (bitmap "SE5.gif"))))

(add-slide 
 26
 (define (slide-26)
   (slide/title
    "Un-Hygienic Macro 'uhm' Template Source"
    'next
    (code
     [(_ body)
      (with-syntax 
          ([utensil 
            (datum->syntax #'body 'utensil)])
        #'(begin
            (define food 'perch)
            (define utensil 'fork)             
            (printf "~a, ~a~n" food utensil)
            body))])
    'next
    (page-item (code with-syntax) "provides the functionality to twist the lexical scope within the macro")
    'next
    (page-item "In this macro," (code utensil) "is inserted into the macro body's environment")
    'next
    (page-item "On the next page, you will see that the macro overrode the existing binding in the body"))))

(add-slide 
 27
 (define (slide-27)
   (slide/title
    "Un-Hygienic Macro 'uhm' Template Expansion"
    'next
    (page-item "[Review code in macro stepper and tracing arrows]")
    'next
    (page-item "Prints \"perch, fork\", then \"salmon, fork\"")
    'next   
    (page-item "The template inserts a new binding into the body for" (code utensil) ", breaking hygiene"))))

(add-slide 
 28
 (define (slide-28)
   (slide
    ; Commented out to conserve space   
    ; "Syntactic Extension 6 - Un-Hygienic Macro Template"
    (bitmap "SE6.gif"))))

(add-slide 
 29
 (define (slide-29)
   (slide/title
    "Next Steps"
    'next
    (page-item "Implement a Primitive Object Syntax")
    'next
    (page-item "Implement Collision Detection")
    'next
    (page-item "Added Default Class Name & Methods Query"))))

(add-slide 
 30
 (define (slide-30)
   (slide/title
    "Prim Obj Stx: Pattern"
    'next
    (code
     (syntax-case stx (variables methods)
       [(define-object name
          (variables ([v-name v-val] ...))
          (methods ([m-name m-args m-body] ...)))]))
    'next
    (page-item "Reference: 4-prim-obj-stx.scm"))))

(add-slide 
 31
 (define (slide-31)
   (slide/title
    "Prim Obj Stx: Template"
    (code
     #'(define name
         (λ () 
           (define class-name (λ () 'name))              
           (define method-names (λ () method-names-ls))
           (define method-names-ls 
             '(class-name method-names m-name ...))
           (define v-name v-val) ...
           (define m-name (λ m-args m-body)) ...
           (λ (msg . args) 
             (case msg                  
               [(class-name) (class-name)]
               [(method-names) (method-names)]
               [(m-name) (apply m-name args)] ...
               [else 
                (raise 'err)]))))))))

(add-slide 
 32
 (define (slide-32)
   (slide/title
    "Prim Obj Stx: Support Code"
    'next
    (page-item "invalid/duplicate identifier detection")
    'next
    (page-subitem "Implementing using" (code identifier?) "and" (code bound-identifier=?)))))  

(add-slide 
 33
 (define (slide-33)
   (slide/title
    "Thoughts"
    'next
    (page-item "Toys are for Learning")
    'next
    (page-item "The H-Word, and Other Hang-Ups")
    'next
    (page-item "Ideas Matter Most, Language Slavery, Innovation")
    'next
    (page-item "CoE: A Perfect \"First Time\"")
    'next
    (page-item "Thoughtful Teacher, Thoughtful Student")
    'next
    (page-item "As Difficult As [I] Make It")
    'next
    (page-item "The Midget vs. the Digits"))))

(add-slide 
 34
 (define (slide-34)
   (slide/title
    "Resources"
    'next
    (page-item "The Scheme Programming Language, Third Edition. R. Kent Dybvig")
    'next
    (page-subitem "Inspiration for this task, the \"K&R\" book for Scheme")
    'next
    (page-item "PLT Scheme v4.02")
    'next
    (page-subitem "mzscheme, DrScheme, Documentation, Discussion List")
    'next
    (page-subitem "This presentation is written in Scheme, see bos-pres.scm and run.bat")
    'next
    (page-subitem "Hit F5 to evalute and work with any code in the REPL")
    'next
    (page-subitem "Use the #scheme module language. All code unit tested")
    'next
    (page-item "Web: Community-Scheme-Wiki, Schematics Scheme Cookbook")
    'next
    (page-item "[A history of] macro systems... http://lists.gnu.org /archive/html/chicken-users/2008-04/msg00013.html"))))

(add-slide 
 35
 (define (slide-35)
   (slide/title
    "Version"
    (page-item "$LastChangedDate: 2008-08-17 11:08:59 -0500 (Sun, 17 Aug 2008) $")
    (page-item "$LastChangedRevision: 2727 $")
    (page-item "$HeadURL: svn://osiris/scheme-bos-clug/tags/2.01/bos-pres.scm $"))))

(show-slides)